﻿using System;
using System.Collections.Generic;
using System.Linq;
using System.Net;
using System.Windows;
using System.Windows.Controls;
using System.Windows.Documents;
using System.Windows.Input;
using System.Windows.Media;
using System.Windows.Media.Animation;
using System.Windows.Shapes;
using ASPItalia.WebSiteGallery.Helper;
using System.Windows.Media.Imaging;
using ASPItalia.WebSiteGallery.Domain;
using System.Threading;

namespace ASPItalia.WebSiteGallery.Silverlight
{
  public partial class GalleryView : UserControl
  {
    GalleryViewModel viewModel = null;
    //public bool GalleryIsLoading;
    //public bool PicturesIsLoading;
    TimerStoryboard timer = new TimerStoryboard(4000);
    private bool IsRunning = false;

    public GalleryView()
    {
      InitializeComponent();

      BitmapImage b = new BitmapImage();
      b.UriSource = new Uri(String.Format("{0}/background/background.jpg", Helper.Helper.BaseUri));
      BackgroundImage.Source = b;

      BitmapImage b1 = new BitmapImage();
      b1.UriSource = new Uri(String.Format("{0}/background/logo.png", Helper.Helper.BaseUri));
      Logo.Source = b1;

      timer.Completed += new CompletedEventHandler(() =>
      {
        viewModel.GoToNextPicture();
      });

      BigPicture.ImageOpened += ((o, e) =>
      {
        if (this.IsRunning)
          timer.Start();
      });

      BigPicture.ImageSizeChanged += new EventHandler<SizeChangedEventArgs>((o, e) =>
      {
      });

      BigPicture.ImageDownloadProgress += new EventHandler<DownloadProgressEventArgs>((o, e) =>
      {
      });

      viewModel = new GalleryViewModel();
      viewModel.GetAlbumCompleted += new EventHandler(viewModel_GetAlbumCompleted);
      viewModel.GetAlbumError += new EventHandler(viewModel_GetAlbumError);
      viewModel.GetAlbumProgressChanged += new EventHandler<DownloadProgressChangedEventArgs>(viewModel_GetAlbumProgressChanged);

      viewModel.GetPicturesCompleted += new EventHandler(viewModel_GetPicturesCompleted);

      this.DataContext = viewModel;
    }

    private void BodyGrid_MouseMove(object sender, MouseEventArgs e)
    { }


    #region Album
    void viewModel_GetAlbumProgressChanged(object sender, DownloadProgressChangedEventArgs e)
    {

    }

    void viewModel_GetAlbumError(object sender, EventArgs e)
    {

    }

    void viewModel_GetAlbumCompleted(object sender, EventArgs e)
    {

    }

    private void AlbumItem_Loaded(object sender, RoutedEventArgs e)
    {
      FrameworkElement elem = sender as FrameworkElement;
      ListBoxItem obj = VisualTreeHelper.GetParent(elem) as ListBoxItem;
      if (obj != null)
      {
        Album item = obj.DataContext as Album;

        double delay = 200 * viewModel.Albums.IndexOf(item);

        new ScaleStoryboard(obj, ScaleStoryboard.ScaleKind.ScaleIn, delay, 95).Start();

        Image img = obj.FindName("Img") as Image;
        if (img != null)
        {
          //se l'immagine è in cache l'evento non si scatena..
          ((BitmapImage)img.Source).DownloadProgress += new EventHandler<DownloadProgressEventArgs>(ImgSource_DownloadProgress);
        }
      }
    }

    void ImgSource_DownloadProgress(object sender, DownloadProgressEventArgs e)
    {
      if (e.Progress == 100)
      {
      }
    }

    private void Img_ImageFailed(object sender, ExceptionRoutedEventArgs e)
    {
      if (((Image)sender).Source != null && e.ErrorException != null)
      {
        //((Image)sender).Source = new BitmapImage(new Uri(string.Format(defaulImageUrl, rootSitePath)));
      }
    }

    private void AlbumListBox_SelectionChanged(object sender, SelectionChangedEventArgs e)
    {
      ListBox obj = sender as ListBox;
      if (obj != null)
      {
        if (obj.SelectedItem != null)
        {
          PictureListBox.SelectedIndex = 0;
          PictureListBox.ScrollIntoView(null);

          Album item = obj.SelectedItem as Album;
          if (item != null)
          {
            ThreadPool.QueueUserWorkItem(viewModel.GetPicturesOfAlbum, item.Title);
          }
        }
      }
    }

    #endregion

    private void PictureItem_Loaded(object sender, RoutedEventArgs e)
    {
      FrameworkElement elem = sender as FrameworkElement;
      ListBoxItem obj = VisualTreeHelper.GetParent(elem) as ListBoxItem;
      if (obj != null)
      {
        Picture item = obj.DataContext as Picture;

        double delay = 100 * viewModel.Pictures.IndexOf(item);

        new ScaleStoryboard(obj, ScaleStoryboard.ScaleKind.ScaleIn, delay, 95).Start();

        Image img = obj.FindName("Img") as Image;
        if (img != null)
        {
          //se l'immagine è in cache l'evento non si scatena..
          ((BitmapImage)img.Source).DownloadProgress += new EventHandler<DownloadProgressEventArgs>(ImgSource_DownloadProgress);
        }
      }
    }

    private void PictureListBox_SelectionChanged(object sender, SelectionChangedEventArgs e)
    {
      ListBox obj = sender as ListBox;
      if (obj != null)
      {
        if (obj.SelectedItem != null)
        {
          Picture item = obj.SelectedItem as Picture;
          if (item != null)
          {
            viewModel.SetCurrentPicture(item);
            PictureListBox.ScrollIntoView(item);
          }
        }
      }
    }

    void viewModel_GetPicturesCompleted(object sender, EventArgs e)
    {
      Dispatcher.BeginInvoke(() => { });
    }

    private void StartSlideShowButton_Checked(object sender, RoutedEventArgs e)
    {
      this.IsRunning = true;
      timer.Start();

      //refact
      BodyGrid.ColumnDefinitions[0].Width = new GridLength(0);
      MediaSpace.Margin = new Thickness(0, 0, 0, 0);
    }

    private void StartSlideShowButton_Unchecked(object sender, RoutedEventArgs e)
    {
      this.IsRunning = false;
      timer.Pause();
      //refact
      BodyGrid.ColumnDefinitions[0].Width = new GridLength(200);
      MediaSpace.Margin = new Thickness(100, 0, 0, 0);
    }

    private void MediaPicture_ImageFailed(object sender, ExceptionRoutedEventArgs e)
    {
    }
  }
}
