﻿using System;
using System.Net;
using System.Windows;
using System.Windows.Controls;
using System.Windows.Documents;
using System.Windows.Ink;
using System.Windows.Input;
using System.Windows.Media;
using System.Windows.Media.Animation;
using System.Windows.Shapes;
using System.Collections.Generic;
using ASPItalia.WebSiteGallery.Domain;
using System.IO;
using System.Xml.Linq;
using System.Linq;
using System.Net.Browser;

namespace ASPItalia.WebSiteGallery.Silverlight
{
  public class GalleryModel : IGalleryModel
  {
    IEnumerable<Album> albums;
    IEnumerable<Picture> pictures;
    const string xmlAlbumPath = "{0}/Albums.ashx";
    const string xmlPicturesOfAlbum = "{0}/PicturesOfAlbum.ashx?albumId={1}";

    public event EventHandler<AlbumLoadedEventArgs> AlbumLoadCompleted;
    public event EventHandler<DownloadProgressChangedEventArgs> GalleryLoadProgressChanged;
    public event EventHandler<GalleryLoadingErrorEventArgs> GalleryLoadingError;
    public event EventHandler<PictureLoadedEventArgs> PictureLoadCompleted;

    #region Album

    public void GetAlbums()
    {
      WebClient albumClient = new WebClient();
      albumClient.Encoding = System.Text.Encoding.UTF8;
      albumClient.DownloadStringCompleted += new DownloadStringCompletedEventHandler(albumClient_DownloadStringCompleted);
      albumClient.DownloadProgressChanged += new DownloadProgressChangedEventHandler(albumClient_DownloadProgressChanged);
      albumClient.DownloadStringAsync(new Uri(String.Format(xmlAlbumPath, Helper.Helper.BaseUri)));
    }

    void albumClient_DownloadProgressChanged(object sender, DownloadProgressChangedEventArgs e)
    {
      OnDownloadProgressChanged(e);
    }

    void albumClient_DownloadStringCompleted(object sender, DownloadStringCompletedEventArgs e)
    {
      if (e.Result != null && e.Error == null && !e.Cancelled)
      {
        string data = e.Result;

        XDocument album = XDocument.Load(new StringReader(data));

        albums = from alb in album.Elements("albums").Elements()
                 select new Album
                 {
                   Title = alb.Element("Title").Value
                     ,
                   Folder = alb.Element("Folder").Value
                     ,
                   ImageFileName = alb.Element("ImageFileName").Value
                 };
        OnAlbumLoadCompleted(albums);
      }
      else
      {
        OnGalleryLoadingError(new Exception("errors"));
      }
    }

    public void OnAlbumLoadCompleted(IEnumerable<Album> result)
    {
      if (AlbumLoadCompleted != null)
        AlbumLoadCompleted(this, new AlbumLoadedEventArgs(result));
    }

    #endregion

    #region Picture
    public void GetPicturesOfAlbum(object albumId)
    {
      WebClient pictureClient = new WebClient();
      pictureClient.Encoding = System.Text.Encoding.UTF8;
      pictureClient.DownloadStringCompleted += new DownloadStringCompletedEventHandler(pictureClient_DownloadStringCompleted);
      pictureClient.DownloadProgressChanged += new DownloadProgressChangedEventHandler(client_DownloadProgressChanged);
      pictureClient.DownloadStringAsync(new Uri(string.Format(xmlPicturesOfAlbum, Helper.Helper.BaseUri, albumId)));
    }

    void pictureClient_DownloadStringCompleted(object sender, DownloadStringCompletedEventArgs e)
    {
      if (e.Result != null && e.Error == null && !e.Cancelled)
      {
        //Thread.Sleep(3000);

        string data = e.Result;

        XDocument picture = XDocument.Load(new StringReader(data));

        pictures = from alb in picture.Elements("pictures").Elements()
                   select new Picture
                   {
                     Title = alb.Element("Title").Value
                       ,
                     ImageFileName = alb.Element("ImageFileName").Value
                       ,
                     AlbumTitle = alb.Element("AlbumTitle").Value
                   };
        OnPictureLoadCompleted(pictures);
      }
      else
      {
        OnGalleryLoadingError(new Exception("errors"));
      }
    }

    public void OnPictureLoadCompleted(IEnumerable<Picture> result)
    {
      if (PictureLoadCompleted != null)
        PictureLoadCompleted(this, new PictureLoadedEventArgs(result));
    }

    void client_DownloadProgressChanged(object sender, DownloadProgressChangedEventArgs e)
    {
      OnDownloadProgressChanged(e);
    }
    #endregion

    #region Generic Event

    public void OnGalleryLoadingError(Exception ex)
    {
      if (GalleryLoadingError != null)
        GalleryLoadingError(this, new GalleryLoadingErrorEventArgs(ex));
    }

    void OnDownloadProgressChanged(DownloadProgressChangedEventArgs e)
    {
      if (GalleryLoadProgressChanged != null)
        GalleryLoadProgressChanged(this, e);
    }
    #endregion
  }
}
