﻿using System;
using System.Net;
using System.Windows;
using System.Windows.Controls;
using System.Windows.Documents;
using System.Windows.Ink;
using System.Windows.Input;
using System.Windows.Media;
using System.Windows.Media.Animation;
using System.Windows.Shapes;
using System.ComponentModel;
using System.Threading;
using System.Collections.ObjectModel;
using ASPItalia.WebSiteGallery.Domain;

namespace ASPItalia.WebSiteGallery.Silverlight
{
  public class GalleryViewModel : INotifyPropertyChanged
  {
      IGalleryModel galleryDataModel = null;
      ObservableCollection<Album> albums = new ObservableCollection<Album>();
      ObservableCollection<Picture> pictures = new ObservableCollection<Picture>();
      Picture currentPicture = null;

      public event EventHandler GetAlbumCompleted;
      public event EventHandler GetAlbumError;
      public event EventHandler<DownloadProgressChangedEventArgs> GetAlbumProgressChanged;

      public event EventHandler GetPicturesCompleted;
      public event EventHandler GetPicturesError;
      public event EventHandler<DownloadProgressChangedEventArgs> GetPicturesProgressChanged;

      public GalleryViewModel() : this(new GalleryModel()) { }

      public GalleryViewModel(IGalleryModel dataModel)
      {
        galleryDataModel = dataModel;
        galleryDataModel.AlbumLoadCompleted += new EventHandler<AlbumLoadedEventArgs>(galleryDataModel_GalleryLoadCompleted);
        galleryDataModel.GalleryLoadingError += new EventHandler<GalleryLoadingErrorEventArgs>(galleryDataModel_GalleryLoadingError);
        galleryDataModel.GalleryLoadProgressChanged += new EventHandler<DownloadProgressChangedEventArgs>(galleryDataModel_GalleryLoadProgressChanged);

        galleryDataModel.PictureLoadCompleted += new EventHandler<PictureLoadedEventArgs>(galleryDataModel_PictureLoadCompleted);

        ThreadPool.QueueUserWorkItem(GetAlbums, null);
      }

      #region Albums
      void galleryDataModel_GalleryLoadCompleted(object sender, AlbumLoadedEventArgs e)
      {
        Deployment.Current.Dispatcher.BeginInvoke(() =>
        {
          //conversione a ObservableCollection 
          foreach (Album item in e.Result)
          {
            albums.Add(item);
          }
          if (GetAlbumCompleted != null)
            GetAlbumCompleted(sender, e);

          if (Albums.Count > 0)
            ThreadPool.QueueUserWorkItem(GetPicturesOfAlbum, Albums[0].Title);
        });
      }

      void galleryDataModel_GalleryLoadingError(object sender, GalleryLoadingErrorEventArgs e)
      {
        if (GetAlbumError != null)
          GetAlbumError(sender, e);
      }

      void galleryDataModel_GalleryLoadProgressChanged(object sender, DownloadProgressChangedEventArgs e)
      {
        if (GetAlbumProgressChanged != null)
          GetAlbumProgressChanged(sender, e);
      }

      public void GetAlbums(object state)
      {
        galleryDataModel.GetAlbums();
      }

      public ObservableCollection<Album> Albums
      {
        get { return albums; }
      }
      #endregion

      #region Pictures

      void galleryDataModel_PictureLoadCompleted(object sender, PictureLoadedEventArgs e)
      {
        Deployment.Current.Dispatcher.BeginInvoke(() =>
        {
          pictures.Clear();

          //conversione a ObservableCollection 
          foreach (Picture item in e.Result)
          {
            pictures.Add(item);

            if (pictures.Count == 1)
              SetCurrentPicture(Pictures[0]);
          }

          if (GetPicturesCompleted != null)
            GetPicturesCompleted(sender, e);

        });
      }

      public void GetPicturesOfAlbum(object albumId)
      {
        galleryDataModel.GetPicturesOfAlbum(albumId);
      }

      public ObservableCollection<Picture> Pictures
      {
        get { return pictures; }
      }

      public void SetCurrentPicture(Picture picture)
      {
        Deployment.Current.Dispatcher.BeginInvoke(() =>
        {
          CurrentPicture = picture;
        });
      }

      public void GoToNextPicture()
      {
        int currentPictureIndex = pictures.IndexOf(CurrentPicture);
        int nextPicture = (currentPictureIndex + 1) % Pictures.Count;
        SetCurrentPicture(Pictures[nextPicture]);
      }

      public Picture CurrentPicture
      {
        get { return currentPicture; }
        set
        {
          currentPicture = value;
          this.OnPropertyChanged("CurrentPicture");
        }
      }
      #endregion

      public event PropertyChangedEventHandler PropertyChanged;
      protected void OnPropertyChanged(string propertyName)
      {
        PropertyChangedEventHandler propertyChanged = this.PropertyChanged;
        if (propertyChanged != null)
          propertyChanged(this, new PropertyChangedEventArgs(propertyName));
      }
    }

}
